@@ -2,13 +2,13 @@ import base64
 import logging
 from abc import ABC, abstractmethod
 from collections import defaultdict
-from typing import Any, Callable, DefaultDict, Dict, Final, List, Optional, Set, Tuple
+from typing import Any, Callable, DefaultDict, Dict, List, Optional, Set, Tuple
 
 from meta_memcache.base.base_serializer import BaseSerializer
-from meta_memcache.base.base_write_failure_tracker import BaseWriteFailureTracker
 from meta_memcache.base.connection_pool import ConnectionPool
 from meta_memcache.base.memcache_socket import MemcacheSocket
 from meta_memcache.errors import MemcacheError, MemcacheServerError
+from meta_memcache.events.write_failure_event import WriteFailureEvent
 from meta_memcache.protocol import (
     ENDL,
     Conflict,
@@ -37,11 +37,10 @@ class BaseCachePool(ABC):
         self,
         serializer: BaseSerializer,
         binary_key_encoding_fn: Callable[[Key], bytes],
-        write_failure_tracker: Optional[BaseWriteFailureTracker] = None,
     ) -> None:
         self._serializer = serializer
         self._binary_key_encoding_fn = binary_key_encoding_fn
-        self._write_failure_tracker: Final = write_failure_tracker
+        self.on_write_failure = WriteFailureEvent()
 
     @abstractmethod
     def _get_pool(self, key: Key) -> ConnectionPool:
@@ -75,11 +74,11 @@ class BaseCachePool(ABC):
         if flags:
             cmd_flags.extend(flag.value for flag in flags)
         if int_flags:
-            for flag, int_value in int_flags.items():
-                cmd_flags.append(flag.value + str(int_value).encode("ascii"))
+            for int_flag, int_value in int_flags.items():
+                cmd_flags.append(int_flag.value + str(int_value).encode("ascii"))
         if token_flags:
-            for flag, bytes_value in token_flags.items():
-                cmd_flags.append(flag.value + bytes_value)
+            for token_flag, bytes_value in token_flags.items():
+                cmd_flags.append(token_flag.value + bytes_value)
         cmd.extend(cmd_flags)
         return b" ".join(cmd) + ENDL
 
@@ -111,9 +110,8 @@ class BaseCachePool(ABC):
                 )
                 return self._conn_recv_response(conn, flags=flags)
         except MemcacheServerError:
-            if self._write_failure_tracker:
-                if command in (MetaCommand.META_DELETE, MetaCommand.META_SET):
-                    self._write_failure_tracker.add_key(key)
+            if command in (MetaCommand.META_DELETE, MetaCommand.META_SET):
+                self.on_write_failure(key)
             raise
 
     def _exec_multi(
@@ -152,12 +150,12 @@ class BaseCachePool(ABC):
                     for key in pool_keys:
                         results[key] = self._conn_recv_response(conn, flags=flags)
         except MemcacheServerError:
-            if self._write_failure_tracker and command in (
+            if command in (
                 MetaCommand.META_DELETE,
                 MetaCommand.META_SET,
             ):
-                for k in keys:
-                    self._write_failure_tracker.add_key(k)
+                for key in keys:
+                    self.on_write_failure(key)
             raise
         return results
 
@@ -214,7 +212,10 @@ class BaseCachePool(ABC):
                 try:
                     result.value = self._serializer.unserialize(data, encoding_id)
                 except Exception:
-                    _log.exception("Error unserializing value")
+                    _log.exception(
+                        f"Error unserializing value {data} "
+                        f"with encoding id: {encoding_id}"
+                    )
                     result = Miss()
 
         return result
@@ -263,7 +264,7 @@ class BaseCachePool(ABC):
     def meta_set(
         self,
         key: Key,
-        value: Any,  # pyre-ignore[2]
+        value: Any,
         ttl: int,
         flags: Optional[Set[Flag]] = None,
         int_flags: Optional[Dict[IntFlag, int]] = None,
